@@ -41,6 +41,7 @@
 #include "pango/cogl-pango.h"
 
 #include "clutter-backend.h"
+#include "clutter-device-manager.h"
 #include "clutter-event.h"
 #include "clutter-feature.h"
 #include "clutter-id-pool.h"
@@ -52,6 +53,8 @@
 
 G_BEGIN_DECLS
 
+typedef struct _ClutterMainContext      ClutterMainContext;
+
 typedef enum {
   CLUTTER_ACTOR_UNUSED_FLAG    = 0,
 
@@ -82,21 +85,54 @@ typedef enum {
 
 struct _ClutterInputDevice
 {
-  gint          id;
+  GObject parent_instance;
+
+  gint id;
 
   ClutterInputDeviceType device_type;
 
-  ClutterActor *pointer_grab_actor;
-  ClutterActor *motion_last_actor;
+  gchar *device_name;
 
-  gint          click_count;
-  gint          previous_x; 
-  gint          previous_y; 
-  guint32       previous_time;
-  gint          previous_button_number;
+  /* the actor underneath the pointer */
+  ClutterActor *cursor_actor;
+
+  /* the actor that has a grab in place for the device */
+  ClutterActor *pointer_grab_actor;
+
+  /* the current click count */
+  gint click_count;
+
+  /* the stage the device is on */
+  ClutterStage *stage;
+
+  /* the current state */
+  gint current_x;
+  gint current_y;
+  guint32 current_time;
+  gint current_button_number;
+  ClutterModifierType current_state;
+
+  /* the previous state, used for click count generation */
+  gint previous_x;
+  gint previous_y;
+  guint32 previous_time;
+  gint previous_button_number;
+  ClutterModifierType previous_state;
 };
 
-typedef struct _ClutterMainContext ClutterMainContext;
+struct _ClutterStageManager
+{
+  GObject parent_instance;
+
+  GSList *stages;
+};
+
+struct _ClutterDeviceManager
+{
+  GObject parent_instance;
+
+  GSList *devices;
+};
 
 struct _ClutterMainContext
 {
@@ -138,9 +174,6 @@ struct _ClutterMainContext
   PangoContext     *pango_context;      /* Global Pango context */
   CoglPangoFontMap *font_map;           /* Global font map */
 
-  GSList              *input_devices;   /* For extra input devices, i.e
-                                           MultiTouch */
-
   ClutterEvent *current_event;
   guint32 last_event_time;
 
@@ -170,21 +203,34 @@ PangoContext *_clutter_context_get_pango_context    (ClutterMainContext *self);
 
 #define I_(str)  (g_intern_static_string ((str)))
 
+/* device manager */
+void _clutter_device_manager_add_device     (ClutterDeviceManager *device_manager,
+                                             ClutterInputDevice   *device);
+void _clutter_device_manager_remove_device  (ClutterDeviceManager *device_manager,
+                                             ClutterInputDevice   *device);
+void _clutter_device_manager_update_devices (ClutterDeviceManager *device_manager);
+
+/* input device */
+void          _clutter_input_device_set_coords (ClutterInputDevice  *device,
+                                                gint                 x,
+                                                gint                 y);
+void          _clutter_input_device_set_state  (ClutterInputDevice  *device,
+                                                ClutterModifierType  state);
+void          _clutter_input_device_set_time   (ClutterInputDevice  *device,
+                                                guint32              time_);
+void          _clutter_input_device_set_stage  (ClutterInputDevice  *device,
+                                                ClutterStage        *stage);
+void          _clutter_input_device_set_actor  (ClutterInputDevice  *device,
+                                                ClutterActor        *actor);
+ClutterActor *_clutter_input_device_update     (ClutterInputDevice  *device);
+
 /* stage manager */
-struct _ClutterStageManager
-{
-  GObject parent_instance;
-
-  GSList *stages;
-};
-
 void _clutter_stage_manager_add_stage    (ClutterStageManager *stage_manager,
                                           ClutterStage        *stage);
 void _clutter_stage_manager_remove_stage (ClutterStageManager *stage_manager,
                                           ClutterStage        *stage);
 
 /* stage */
-
 void                _clutter_stage_set_window           (ClutterStage       *stage,
                                                          ClutterStageWindow *stage_window);
 ClutterStageWindow *_clutter_stage_get_window           (ClutterStage       *stage);
@@ -198,6 +244,7 @@ void     _clutter_stage_queue_event           (ClutterStage *stage,
 					       ClutterEvent *event);
 gboolean _clutter_stage_has_queued_events     (ClutterStage *stage);
 void     _clutter_stage_process_queued_events (ClutterStage *stage);
+void     _clutter_stage_update_input_devices  (ClutterStage *stage);
 
 /* vfuncs implemented by backend */
 GType         _clutter_backend_impl_get_type  (void);
@@ -271,6 +318,9 @@ void _clutter_actor_set_enable_model_view_transform (ClutterActor *self,
 void _clutter_actor_set_enable_paint_unmapped (ClutterActor *self,
                                                gboolean      enable);
 
+void _clutter_actor_set_has_pointer (ClutterActor *self,
+                                     gboolean      has_pointer);
+
 void _clutter_run_repaint_functions (void);
 
 gint32 _clutter_backend_get_units_serial (ClutterBackend *backend);
